Attribute VB_Name = "modAddReplaceCode"
Option Compare Database
Option Explicit

'---------------------------------------------------------------------------------------
' Module    : modAddReplaceCode
' DateTime  : 22/01/2023
' Author    : Colin Riddington (Mendip Data Systems)
' Website   : https://www.isladogs.co.uk
' Purpose   : Find/replace/remove specified code in selected module
' Copyright : The code in the utility MAY be altered and reused in your own applications
'             provided the copyright notice is left unchanged (including Author, Website and Copyright)
'             You are NOT allowed to sell, resell or repost this on other sites such as online forums
'             without permission from the author. However, links back to the above websites ARE allowed.

'             If you find this code useful please place a link to my website on your own web site
'             so that others may benefit as well.
' Updated   : January 2023
'---------------------------------------------------------------------------------------

Dim strText As String, strText2 As String, strCode As String
Dim rst As DAO.Recordset, mdl As Module
Dim strSQL As String, strName As String, strModuleName As String
Dim N As Integer

Sub AddFormLoadResizeCode()

On Error GoTo Err_Handler

    'check for form objects
    N = DCount("*", "MSysObjects", "MSysObjects.Type =-32768")

    strSQL = "SELECT MSysObjects.Name FROM MSysObjects" & _
        " WHERE MSysObjects.Type =-32768 ORDER BY MSysObjects.Name;"
    
    Set rst = CurrentDb.OpenRecordset(strSQL)
    
    'loop through each form and add resizing code if it doesn't already exist
    Do Until rst.EOF
         strName = rst("Name")
         strModuleName = "Form_" & strName
        
         'open form in design view so it can be modified
        DoCmd.OpenForm strName, acDesign, , , , acHidden
        
        Set mdl = Forms(strName).Module
        
           'set Form_Load code
            strText = "ResizeForm Me"
            strText2 = "'MaintainAspectRatio 'enable to stretch/shrink"
            
             'set Form_Load code - use different code for forms with datasheet subforms
            If strName = "Customer Orders" Or strName = "Orders" Then 'these have DS subforms
                strCode = "" & vbCrLf & "Private Sub Form_Load()" & _
                    vbCrLf & vbTab & "If Not (IsFormDialog(Me) Or IsSubform(Me)) Then GetOriginalSize Me" & _
                    vbCrLf & vbTab & strText & vbCrLf & vbTab & strText2 & vbCrLf & "End Sub"
            Else
                strCode = "" & vbCrLf & "Private Sub Form_Load()" & _
                    vbCrLf & vbTab & "If Not (IsFormDialog(Me) Or IsSubform(Me)) Then GetOriginalSize Me" & _
                    vbCrLf & vbTab & strText & vbCrLf & "End Sub"
            End If
    
             'add Form_Load code if not already done
            If CodeFind(strModuleName, strText) = False Then
                   mdl.InsertText strCode
            End If
            
            'set Form_Resize code
            strText = "If Not (Me.Modal Or IsFormDialog(Me) Or IsSubform(Me)) Then StretchShrink Me"
            'use different code for forms with datasheet subforms
            strText2 = "'MaintainAspectRatio  'enable to stretch/shrink"
            
            If strName = "Customer Orders" Or strName = "Orders" Then 'these have DS subforms
                strCode = "" & vbCrLf & "Private Sub Form_Resize()" & vbCrLf & vbTab & strText2 & vbCrLf & "End Sub"
            Else
                strCode = "" & vbCrLf & "Private Sub Form_Resize()" & vbCrLf & vbTab & strText & vbCrLf & "End Sub"
            End If
                
             'add Form_Resize code if not already done
            If CodeFind(strModuleName, strText) = False Then
                   mdl.InsertText strCode
            End If
          
        'save & close form
        DoCmd.Close acForm, strName, acSaveYes
       
    rst.MoveNext
    Loop
    rst.Close
    
    Set rst = Nothing
    
      'save and compile all code modules
    SysCmd 504, 16483
      'this is equivalent to
      'DoCmd.RunCommand acCmdCompileAndSaveAllModules

     'end message
    MsgBox "All " & N & " forms in the Northwind 2000 database" & _
            " have been updated with automatic form resizing code" & vbCrLf & vbCrLf & _
            "======================================" & vbCrLf & vbCrLf & _
            "*NOTE:**" & vbCrLf & _
            "Add MaintainAspectRatio sub for use with DS subforms", vbInformation, "AFR successfully added"

Exit_Handler:
    Exit Sub
    
Err_Handler:
    MsgBox "Error " & Err & " : " & Err.Description & " in AddFormLoadCode procedure"
    Resume Exit_Handler
    
End Sub

Sub FormWithDatasheetSubformCode()

'add this code to Form_Load where form has a datasheet subform
'i.e. Customer_Orders (x2) & Orders (NOT Quarterly_Orders as continuous subform)
'replace 'SubformName' with actual name of subform control

    Dim ctl As Control
    Me.SubformName.Form.DatasheetFontHeight = 8 * GetFactor

    For Each ctl In Me.SubformName.Form.Controls
          'loop through controls and adjust column widths automatically
            Select Case ctl.ControlType
              Case acTextBox, acCheckBox, acComboBox 'limit to prevent error 438
                    ctl.ColumnWidth = -2

            End Select
    Next
    
End Sub

Private Sub Form_Resize()
  ' ' If Not (Me.Modal Or IsFormDialog(Me) Or IsSubform(Me)) Then StretchShrink Me
    
  '  MaintainAspectRatio
End Sub

Sub MaintainAspectRatio()

'---------------------------------------------------------------------------------------
' Procedure :            MaintainAspectRatio
' DateTime  :           23/12/2022
' Author :                 Colin Riddington
' Purpose :               Use code directly in a form with datasheet subform to maintain width:height ratio when the form is resized using a mouse
'                               DO NOT RUN FROM THIS MODULE
'---------------------------------------------------------------------------------------

On Error Resume Next
    
    Dim sngStretchWidth As String, sngStretchHeight As String
    Dim blnRestore As Boolean

    Application.Echo False

    'ensures width & height are changed in proportion as form is stretched / shrunk
    GetResizedWindow Me

    sngStretchWidth = lngNewWidth / lngOldWidth
    sngStretchHeight = lngNewHeight / lngOldHeight

    If sngStretchWidth >= sngStretchHeight Then
        If sngStretchWidth >= 1 Then
            sngStretchHeight = sngStretchWidth
            lngNewHeight = lngOldHeight * sngStretchHeight
        Else
            sngStretchWidth = sngStretchHeight
            lngNewWidth = lngOldWidth * sngStretchWidth
        End If

    Else
        If sngStretchHeight >= 1 Then
            sngStretchWidth = sngStretchHeight
            lngNewWidth = lngOldWidth * sngStretchWidth
        Else
            sngStretchHeight = sngStretchWidth
            lngNewHeight = lngOldHeight * sngStretchHeight
        End If

    End If

    DoEvents
    'move to top left & update form size
    DoCmd.MoveSize 0, 0, lngNewWidth, lngNewHeight

    'adjust height of each section in proportion
    If GetStretchFactor <> 1 Then 'resize necessary
       Me.Detail.Height = lngDetailHeight * GetStretchFactor
    '   Me.FormHeader.Height = lngHeaderHeight * GetStretchFactor
    '   Me.FormFooter.Height = lngFooterHeight * GetStretchFactor

       'resize all controls
       Resize GetStretchFactor, Me

     '  Resize GetStretchFactor, Customer_Order_Subform1.Form
       
           Dim ctl As Control
           '**********REPLACE 'Subform Name' with actual datasheet subform name below (x3)*************
           Me.SubformName.Form.DatasheetFontHeight = Me.SubformName.Form.DatasheetFontHeight * GetStretchFactor
    
          For Each ctl In Me.SubformName.Form.Controls
              'loop through controls and adjust column widths automatically
                Select Case ctl.ControlType
                  Case acTextBox, acCheckBox, acComboBox 'limit to prevent error 438
                        ctl.ColumnWidth = -2
    
                End Select
                
         Next
    End If

    'set old values to equal new values (used in GetStretchFactor)
    lngOldWidth = lngNewWidth
    lngOldHeight = lngNewHeight

    Application.Echo True

End Sub

Sub FormResizeCode()

'Add the following code to the Form_Load event
'========================================================
'Private Sub Form_Load()
'   ReSizeForm Me
'End Sub

End Sub

Sub FormResizeStretchCode()

'Add the following code to the Form_Load event with ResizeForm Me at end
'========================================================
'Private Sub Form_Load()
'   If Not (IsFormDialog(Me) Or IsSubform(Me)) Then GetOriginalSize Me
'   ReSizeForm Me
'End Sub
'
''Add the following code to Form_Resize event
'Private Sub Form_Resize()
'     If Not (Me.Modal Or IsFormDialog(Me) Or IsSubform(Me)) Then StretchShrink Me
'End Sub
'=========================================================

End Sub

Function CodeFind(strModuleName As String, strSearchText As String) As Boolean
          
10    On Error GoTo Err_Handler

      'Form/report MUST first be opened in design view

20        CodeFind = False
          
          Dim mdl As Module
          Dim lngSLine As Long, lngSCol As Long
          Dim lngELine As Long, lngECol As Long
          
          ' Open module.
30        DoCmd.OpenModule strModuleName
          ' Return reference to Module object.
40        Set mdl = Modules(strModuleName)
          
          ' Search for string.
50        If mdl.Find(strSearchText, lngSLine, lngSCol, lngELine, lngECol) Then
60            CodeFind = True
70        End If
       
Exit_Handler:
80        Exit Function
       
Err_Handler:
           'err 2517 - cannot find module
90        If Err = 2517 Then Exit Function
100       MsgBox "Error " & Err.Number & " in line " & Erl & " of Find procedure : " & vbNewLine & "   - " & Err.Description
110       CodeFind = False
120       Resume Exit_Handler
          
End Function

Function CodeFindReplace(strModuleName As String, _
    strSearchText As String, strNewText As String) As Boolean
          
10    On Error GoTo Err_Handler

      'Form/report MUST first be opened in design view
          
          Dim mdl As Module
          Dim lngSLine As Long, lngSCol As Long
          Dim lngELine As Long, lngECol As Long
          Dim strLine As String, strNewLine As String
          Dim intChr As Integer, intBefore As Integer, _
          intAfter As Integer
          Dim strLeft As String, strRight As String
          
          ' Open module.
20        DoCmd.OpenModule strModuleName
          ' Return reference to Module object.
30        Set mdl = Modules(strModuleName)
          
          ' Search for string.
40        If mdl.Find(strSearchText, lngSLine, lngSCol, lngELine, lngECol) Then

              ' Store text of line containing string.
50            strLine = mdl.Lines(lngSLine, Abs(lngELine - lngSLine) + 1)

              ' Determine length of line.
60            intChr = Len(strLine)
              
              ' Determine number of characters preceding search text.
70            intBefore = lngSCol - 1
              
              ' Determine number of characters following search text.
80            intAfter = intChr - CInt(lngECol - 1)
              
              ' Store characters to left of search text.
90            strLeft = Left$(strLine, intBefore)
              
              ' Store characters to right of search text.
100           strRight = Right$(strLine, intAfter)
              
              ' Construct string with replacement text.
110           strNewLine = strLeft & strNewText & strRight
              
              ' Replace original line.
120           mdl.ReplaceLine lngSLine, strNewLine
130           CodeFindReplace = True
140       Else
150           CodeFindReplace = False
160       End If
       
Exit_Handler:
170       Exit Function
       
Err_Handler:
          'err 2517 - cannot find module
180       If Err = 2517 Then Exit Function
190       MsgBox "Error " & Err.Number & " in line " & Erl & " of CodeFindReplace procedure : " & vbNewLine & "   - " & Err.Description
200       CodeFindReplace = False
210       Resume Exit_Handler
          
End Function

Function RemoveCodeLine(strModuleName As String, strSearchText As String) As Boolean
          
10    On Error GoTo Err_Handler

      'Form/report MUST first be opened in design view
          Dim mdl As Module
          Dim lngSLine As Long, lngSCol As Long
          Dim lngELine As Long, lngECol As Long
          Dim strLine As String, strNewLine As String
          
          ' Open module.
20        DoCmd.OpenModule strModuleName
          ' Return reference to Module object.
30        Set mdl = Modules(strModuleName)
          
          ' Search for string.
40        If mdl.Find(strSearchText, lngSLine, lngSCol, lngELine, lngECol) Then

              ' Store text of line containing string.
50            strLine = mdl.Lines(lngSLine, Abs(lngELine - lngSLine) + 1)
              
              ' Remove original line & empty line before
60            mdl.DeleteLines lngSLine - 1, 2
70            RemoveCodeLine = True
80        Else
90            RemoveCodeLine = False
100       End If
       
Exit_Handler:
110       Exit Function
       
Err_Handler:
            'err 2517 - cannot find module
120       If Err = 2517 Then Exit Function
130       MsgBox "Error " & Err.Number & " in line " & Erl & " of RemoveCodeLine procedure : " & vbNewLine & "   - " & Err.Description
140       RemoveCodeLine = False
150       Resume Exit_Handler
          
End Function


